#ifndef _COMM_LINK_ABSTRACT_H
#define _COMM_LINK_ABSTRACT_H

/**
 * @file comm-link-abstract.h
 *
 * Abstraction de la liaison entre un micro-contrôleur et un module d'affichage.
 *
 * Définit le contrat à implémenter pour un mode de communication précis.
 */

#include <stdbool.h>
#include <stdint.h>

/**
 * Type de donnée à lire/écrire.
 */
typedef enum {
	TYPE_COMMAND = 0,/**< Fonctions du contrôleur. */
	TYPE_DATA = 1,   /**< Données de la mémoire d'affichage du contrôleur. */
} CommLinkDataType;

/**
 * Cette structure regroupe les fonctions à implémenter pour être en mesure de
 * communiquer avec un module OLED via une méthode spécifique (ex. I2C, SPI).
 */
typedef struct {
	/**
	 * Initialisations spécifiques de la liaison, le cas échéant.
	 *
	 * @param linkConfig Pointeur sur la structure de configuration de la liaison.
	 */
	void (*initialise)(const void *linkConfig);
	/**
	 * Fonction d'écriture.
	 *
	 * @param linkConfig Pointeur sur la structure de configuration de la liaison.
	 * @param dataType Définit si on transmet une commande ou une donnée.
	 * @param data Valeurs à transmettre.
	 * @param count Nombre d'octets à transmettre.
	 */
	void (*write)(const void *linkConfig, CommLinkDataType dataType, const uint8_t *data, int count);
	/**
	 * Fonction de lecture (données seulement, pas commandes).
	 *
	 * @param linkConfig Pointeur sur la structure de configuration de la liaison.
	 * @param dataType Définit si on lit une commande ou une donnée.
	 * @param buffer Tampon de stockage du résultat.
	 * @param count Nombre d'octets à lire.
	 */
	void (*read)(const void *linkConfig, CommLinkDataType dataType, uint8_t *buffer, int count);
	/**
	 * Fait passer la broche de RESET du module à l'état actif.
	 *
	 * @param linkConfig Pointeur sur la structure de configuration de la liaison.
	 */
	void (*assertReset)(const void *linkConfig);
	/**
	 * Fait passer la broche de RESET du module à l'état inactif.
	 *
	 * @param linkConfig Pointeur sur la structure de configuration de la liaison.
	 */
	void (*deassertReset)(const void *linkConfig);
	/**
	 * Indique si le module utilise une broche de RESET.
	 *
	 * @param linkConfig Pointeur sur la structure de configuration de la liaison.
	 */
	bool (*hasReset)(const void *linkConfig);
	/**
	 * Fonction d'écriture directe, utilisée lors de l'initialisation de certains modules..
	 *
	 * @param linkConfig Pointeur sur la structure de configuration de la liaison.
	 * @param dataType Définit si on transmet une commande ou une donnée.
	 * @param data Valeurs à transmettre.
	 * @param count Nombre d'octets à transmettre.
	 */
	void (*rawWrite)(const void *linkConfig, CommLinkDataType dataType, const uint8_t *data, int count);
	/**
	 * Retourne la largeur du bus de données.
	 * Normalement 8 bits, sauf lors de communication en mode parallèle
	 * sur 4 bits.
	 * @param linkConfig
	 * @return
	 */
	int (*getBusWidth)(const void *linkConfig);
} CommLinkImpl;

/**
 * Structure regroupant les méthodes ("implementation") et les attributs
 * ("configuration") d'un lien de communication concret.
 */
typedef struct {
	const CommLinkImpl *implementation;
	const void *configuration;
} CommLink;

// Les fonctions inline ci-dessous servent uniquement à améliorer la lisibilité du code.

__attribute__((always_inline))
inline void commLinkInitialise(const CommLink *link) {
	link->implementation->initialise(link->configuration);
}

__attribute__((always_inline))
inline void commLinkWrite(const CommLink *link, CommLinkDataType dataType, const uint8_t *data, int count) {
	link->implementation->write(link->configuration, dataType, data, count);
}

__attribute__((always_inline))
inline void commLinkRead(const CommLink *link, CommLinkDataType dataType, uint8_t *buffer, int count) {
	link->implementation->read(link->configuration, dataType, buffer, count);
}

__attribute__((always_inline))
inline void commLinkAssertReset(const CommLink *link) {
	link->implementation->assertReset(link->configuration);
}

__attribute__((always_inline))
inline void commLinkDeassertReset(const CommLink *link) {
	link->implementation->deassertReset(link->configuration);
}

__attribute__((always_inline))
inline bool commLinkHasReset(const CommLink *link) {
	return link->implementation->hasReset(link->configuration);
}

__attribute__((always_inline))
inline void commLinkRawWrite(const CommLink *link, CommLinkDataType dataType, const uint8_t *data, int count) {
	link->implementation->rawWrite(link->configuration, dataType, data, count);
}

__attribute__((always_inline))
inline int commLinkGetBusWidth(const CommLink *link) {
	return link->implementation->getBusWidth(link->configuration);
}

#endif // _COMM_LINK_ABSTRACT_H
