#include "project-defs.h"
#include <stdbool.h>
#define _COMM_LINK_PCF8574_C
#include "comm-link-pcf8574.h"

static void initialise(const void *linkConfig) {
	// L'initialisation de l'interface I2C a déjà été faite dans main.c.
}
/*
 * Relation entre les lignes de port du PCF8574 et les lignes de l'interface
 * parallèle d'un module LCD à base de HD44780 ou équivalent :
 *
 * P7 = D7
 * P6 = D6
 * P5 = D5
 * P4 = D4
 * P3 = Backlight Enable (état haut => LED du backlight allumée)
 * P2 = E (prend en compte les données sur front descendant)
 * P1 = RW (état bas => écriture, état haut => lecture)
 * P0 = RS (état bas => commande, état haut => donnée)
 */

// Backlight enable
#define MASK_BL 0x08
// Ligne E
#define MASK_EN 0x04
// Ligne RW
#define MASK_RW 0x02
// Ligne RS
#define MASK_RS 0x01

// Envoie un octet au module LCD
static void _i2cWrite(int address, uint8_t data) {
	while (I2C_GetFlagStatus(I2C1, I2C_FLAG_BUSY) != RESET);

	I2C_GenerateSTART(I2C1, ENABLE);
	while (!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_MODE_SELECT));

	I2C_Send7bitAddress(I2C1, address << 1, I2C_Direction_Transmitter);
	while (!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_TRANSMITTER_MODE_SELECTED));

	I2C_SendData(I2C1, data);
	while (!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_BYTE_TRANSMITTED));

	I2C_GenerateSTOP(I2C1, ENABLE);
}

static void _write(const CommLinkPCF8574Config *cfg, CommLinkDataType dataType, const uint8_t *data, int count, bool rawWrite) {
	for (int i = 0; i < count; i++) {
		uint8_t byteValue = data[i];
		// On envoie le quartet de poids fort.
		uint8_t data = (byteValue & 0xf0) | MASK_BL | MASK_EN | dataType;
		_i2cWrite(cfg->i2cAddress, data);

		if (!rawWrite) {
			// Le contrôleur LCD prend en compte la donnée sur le
			// front descendant de la ligne E.
			_i2cWrite(cfg->i2cAddress, data & ~MASK_EN);
			// On envoie maintenant le quartet de poids faible.
			data = (byteValue << 4) | MASK_BL | MASK_EN | dataType;
			_i2cWrite(cfg->i2cAddress, data);
		}

		// Le contrôleur LCD prend en compte la donnée sur le
		// front descendant de la ligne E.
		_i2cWrite(cfg->i2cAddress, data & ~MASK_EN);
	}
}

static void write(const void *linkConfig, CommLinkDataType dataType, const uint8_t *data, int count) {
	_write((const CommLinkPCF8574Config *) linkConfig, dataType, data, count, false);
}

static bool hasReset(const void *linkConfig) {
	// Non applicable
	return false;
}

static void assertReset(const void *linkConfig) {
	// Non applicable
}

static void deassertReset(const void *linkConfig) {
	// Non applicable
}

static void read(const void *linkConfig, CommLinkDataType dataType, uint8_t *buffer, int count) {
	// Non applicable
}

static void rawWrite(const void *linkConfig, CommLinkDataType dataType, const uint8_t *data, int count) {
	_write((const CommLinkPCF8574Config *) linkConfig, dataType, data, count, true);
}

static int getBusWidth(const void *linkConfig) {
	return 4;
}

const CommLinkImpl COMM_LINK_PCF8574_IMPL = {
	.initialise = initialise,
	.write = write,
	.read = read,
	.hasReset = hasReset,
	.assertReset = assertReset,
	.deassertReset = deassertReset,
	.rawWrite = rawWrite,
	.getBusWidth = getBusWidth,
};
