#include <comm-link-pcf8574.h>
#include "project-defs.h"
#include <stdbool.h>
#include "txt-module.h"
#include "txt-splc780-controller.h"

// L'adresse par défaut de l'adaptateur I2C du LCD2004 est 0x27 si le circuit
// utilisé est un PCF8574T, ou 0x3f si c'est un PCF8574AT. En dehors de leur
// adresse, les 2 circuits sont identiques.
const CommLinkPCF8574Config COMM_PCF8574_CONFIG = { .i2cAddress = 0x27, };

TxtModule txtModule = {
	// L'affichage comporte 4 lignes de 20 caractères.
	.display = { .width = 20, .height = 4, },
	// Le contrôleur est un SPLC780.
	.controller = &TXT_SPLC780_CONTROLLER,
	// La liaison entre micro-contrôleur et module LCD est un bus I2C.
	.link = { .implementation = &COMM_LINK_PCF8574_IMPL, .configuration = &COMM_PCF8574_CONFIG, },
};

// Le PCF8574 est limité à 100kHz.
#define I2C_CLOCK_FREQ 100000
// On assigne l'adresse I2C 1 au CH32V003.
#define I2C_HOST_ADDR 1

volatile bool buttonPressed = false;

__attribute__((interrupt("WCH-Interrupt-fast")))
void EXTI7_0_IRQHandler() {
	if (EXTI_GetITStatus(EXTI_Line2) != RESET) {
		buttonPressed = true;
		EXTI_ClearITPendingBit(EXTI_Line2);
	}
}

int main() {
	// Nécessaire pour le calcul des délais.
	SystemCoreClockUpdate();
	Delay_Init();
	
	// Nécessaire pour pouvoir utiliser printf().
	USART_Printf_Init(115200);

	/*
	 * IMPORTANT : on laisse l'initialisation de l'interface I2C ici car
	 * il pourrait y avoir plusieurs périphériques I2C connectés au bus.
	 * Cela signifie que cette initialisation n'a pas sa place dans le
	 * code dédié à la communication avec le module LCD.
	 *
	 * Tout le code qui suit est identique à celui du cours I2C et EEPROM,
	 * les changements débutent à la ligne txtInitialise(&txtModule);
	 */

	RCC_APB2PeriphClockCmd(RCC_APB2Periph_AFIO, ENABLE);

	GPIO_InitTypeDef gpioInit = { 0 };
	// PD2 = bouton poussoir
	RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOD, ENABLE);
	gpioInit.GPIO_Pin = GPIO_Pin_2;
	gpioInit.GPIO_Mode = GPIO_Mode_IPU;
	GPIO_Init(GPIOD, &gpioInit);
	// PC1 = SDA
	// PC2 = SCL
	RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOC, ENABLE);
	gpioInit.GPIO_Pin = GPIO_Pin_1 | GPIO_Pin_2;
	gpioInit.GPIO_Mode = GPIO_Mode_AF_OD;
	gpioInit.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_Init(GPIOC, &gpioInit);

	GPIO_EXTILineConfig(GPIO_PortSourceGPIOD, GPIO_PinSource2);

	EXTI_InitTypeDef extiInit = { 0 };
	extiInit.EXTI_Line = EXTI_Line2;
	extiInit.EXTI_Mode = EXTI_Mode_Interrupt;
	extiInit.EXTI_Trigger = EXTI_Trigger_Falling;
	extiInit.EXTI_LineCmd = ENABLE;
	EXTI_Init(&extiInit);

	NVIC_InitTypeDef nvicInit = { 0 };
	nvicInit.NVIC_IRQChannel = EXTI7_0_IRQn;
	nvicInit.NVIC_IRQChannelPreemptionPriority = 0;
	nvicInit.NVIC_IRQChannelSubPriority = 1;
	nvicInit.NVIC_IRQChannelCmd = ENABLE;
	NVIC_Init(&nvicInit);

	RCC_APB1PeriphClockCmd(RCC_APB1Periph_I2C1, ENABLE);
	I2C_InitTypeDef i2cInit = { 0 };
	i2cInit.I2C_ClockSpeed = I2C_CLOCK_FREQ;
	i2cInit.I2C_Mode = I2C_Mode_I2C;
	i2cInit.I2C_DutyCycle = I2C_DutyCycle_2;
	i2cInit.I2C_OwnAddress1 = I2C_HOST_ADDR << 1;
	i2cInit.I2C_Ack = I2C_Ack_Enable;
	i2cInit.I2C_AcknowledgedAddress = I2C_AcknowledgedAddress_7bit;
	I2C_Init(I2C1, &i2cInit);
	I2C_Cmd(I2C1, ENABLE);

	// On initialise le module LCD
	txtInitialise(&txtModule);

	while (1) {
		txtPrintAt(&txtModule, 0, 0, "Hello, World!");
		Delay_Ms(500);
		txtPrintAt(&txtModule, 4, 1, "Tagada!");
		Delay_Ms(500);
		txtPrintAt(&txtModule, 8, 2, "Tsoin-tsoin.");
		Delay_Ms(500);
		txtPrintAt(&txtModule, 12, 3, "The end.");
		Delay_Ms(500);
		txtClear(&txtModule);
		Delay_Ms(500);
	}
/*
	// On affiche un premier message
	txtPrintAt(&txtModule, 0, 0, "Hello, World!");

	int count = 0;

	while (1) {
		// A chaque fois qu'on appuie sur le bouton poussoir,
		if (buttonPressed) {
			buttonPressed = false;
			// on affiche la valeur du compteur et on l'incrémente.
			txtPrintAt(&txtModule, 4, 1, "count: %d", count++);
		}
	}
*/
}
