#ifndef _TXT_ABSTRACT_CONTROLLER_H
#define _TXT_ABSTRACT_CONTROLLER_H

/**
 * @file txt-abstract-controller.h
 *
 * Abstraction de contrôleur d'affichage en mode texte.
 * Définit le contrat à implémenter pour un contrôleur précis.
 */

#include <stdbool.h>
#include "txt-module.h"

/**
 * Cette structure regroupe les fonctions que tout contrôleur doit implémenter
 * pour qu'on soit en mesure d'utiliser un module d'affichage en mode texte.
 */
struct TxtController {
	/**
	 * Initialise le contrôleur.
	 */
	void (*initialise)(TxtModule *module);

	/**
	 * Efface l'écran et positionne le curseur en haut à gauche.
	 */
	void (*clear)(TxtModule *module);

	/**
	 * Positionne le curseur en haut à gauche.
	 */
	void (*returnHome)(TxtModule *module);

	/**
	 * @param textDirection true = Décaler l'affichage quand le curseur avance au dela du bord de l'écran,
	 * false = Ne pas décaler
	 * @param shiftDisplay true =  = quand on incrémente l'adresse, le curseur se déplace à droite ou l'affichage se décale à gauche
	 * (texte LTR, écriture de gauche à droite),
	 * false = quand on décrémente l'adresse, le curseur se déplace à gauche ou l'affichage se décale à droite
	 * (texte RTL, écriture de droite à gauche).
	 */
	void (*setEntryMode)(TxtModule *module, bool textDirection, bool shiftDisplay);

	/**
	 * @param displayOn Mise en service de l'écran on/off
	 * @param cursorOn Affichage du curseur on/off
	 * @param blinkCursor Curseur clignotant on/off
	 */
	void (*displayControl)(TxtModule *module, bool displayOn, bool cursorOn, bool blinkCursor);

	/**
	 * @param shiftDisplay true = Décale l'affichage, false = Déplace le curseur
	 * @param shiftRight true = L'affichage se décale ou le curseur se déplace vers la droite,
	 * false = L'affichage se décale ou le curseur se déplace vers la gauche
	 *
	 * shiftDisplay && shiftRight => L'affichage se décale à droite, le curseur suit, l'adresse ne change pas.
	 * shiftDisplay && !shiftRight => L'affichage se décale à gauche, le curseur suit, l'adresse ne change pas.
	 * !shiftDisplay && shiftRight => Le curseur se déplace à droite, l'adresse est incrementée.
	 * !shiftDisplay && !shiftRight => Le curseur se déplace à gauche, l'adresse est décrementée.
	 *
	 * La configuration que vous voulez est sans doute shiftDisplay = false et
	 * shiftRight = true, c'est-à-dire la configuration par défaut. :)
	 */
	void (*cursorShiftControl)(TxtModule *module, bool shiftDisplay, bool shiftRight);

	/**
	 * Fixe l'adresse d'écriture dans la RAM du générateur de caractères.
	 * A utiliser en conjonction avec writeByte() si vous voulez définir
	 * des caractères sur-mesure pour les codes de 0x00 à 0x0f. Voir la
	 * data sheet pour plus de détails.
	 *
	 * @param module
	 * @param address
	 */
	void (*setCGAddress)(TxtModule *module, int address);

	void (*setCursorPosition)(TxtModule *module, int column, int row);

	void (*writeByte)(TxtModule *module, uint8_t byte);
};

#endif // _TXT_ABSTRACT_CONTROLLER_H
