#include "project-defs.h"
#define _TXT_SPLC780_CONTROLLER_C
#include "txt-splc780-controller.h"

// Temps de traitement en microsecondes de tous les octets envoyés au
// contrôleur. Un délai de traitement supplémentaire s'y ajoute pour
// les fonctions clear() et returnHome().
#define WRITE_DELAY 50

static void _sendRaw(TxtModule *module, uint8_t command) {
	commLinkRawWrite(&module->link, TYPE_COMMAND, &command, 1);
	Delay_Us(WRITE_DELAY);
}

static void _sendCommand(TxtModule *module, uint8_t command) {
	commLinkWrite(&module->link, TYPE_COMMAND, &command, 1);
	Delay_Us(WRITE_DELAY);
}

static void setEntryMode(TxtModule *module, bool textDirection, bool shiftDisplay) {
	uint8_t s = shiftDisplay ? 1 : 0;
	uint8_t id = textDirection ? 2 : 0;
	_sendCommand(module, 0x04 | id | s);
}

static void displayControl(TxtModule *module, bool displayOn, bool cursorOn, bool blinkCursor) {
	uint8_t d = displayOn ? 4 : 0;
	uint8_t c = cursorOn ? 2 : 0;
	uint8_t s = blinkCursor ? 1 : 0;
	_sendCommand(module, 0x08 | d | c | s);
}

static void clear(TxtModule *module) {
	_sendCommand(module, 0x01);
	// Cette opération dure 1.6ms mais nous avons déjà attendu WRITE_DELAY
	// dans _sendCommand()
	Delay_Us(1600 - WRITE_DELAY);
}

static void initialise(TxtModule *module) {
	// La data sheet demande d'attendre au moins 50ms
	Delay_Us(50000);

	// -- On configure l'interface parallèle

	// Dans le cas d'une réinitialisation à chaud, on doit s'assurer que
	// l'interface parallèle soit en mode 8 bits afin d'être dans la même
	// situation que lors d'une initialisation à froid (à la mise sous tension).
	_sendRaw(module, 0x30);
	// Attendre 4.1ms, mais nous avons déjà attendu WRITE_DELAY dans _sendCommnand()
	Delay_Us(4100 - WRITE_DELAY);
	_sendRaw(module, 0x30);
	// Attendre 100us, mais nous avons déjà attendu WRITE_DELAY dans _sendCommnand()
	Delay_Us(100 - WRITE_DELAY);
	_sendRaw(module, 0x30);

	int busWidth = commLinkGetBusWidth(&module->link);

	if (busWidth != 8) {
		// On passe en mode 4 bits
		_sendRaw(module, 0x20);
	}

	// -- On initialise l'affichage

	// Commande "Function Set" (valide seulement à l'initialisation)
	// dl = nombre de bits de données de l'interface (0 = 4 bits, 0x10 = 8 bits)
	uint8_t dl = (busWidth != 8) ? 0 : 0x10;
	// n = nombre de lignes de l'affichage (0 = 1 line, 0x08 = 2 lines)
	uint8_t n = module->display.height > 1 ? 0x08 : 0;
	// f = police de caractères (0 = 5x8 pixels, 0x04 = 5x10 pixels)
	// Remarque: on ne peut pas afficher sur 2 lignes avec la police 5x10.
	uint8_t f = 0;
	_sendCommand(module, 0x20 | dl | n | f);

	// Met en fonction l'affichage, pas de curseur, pas de clignotement.
	displayControl(module, true, false, false);

	// Efface l'affichage
	clear(module);

	// Set Entry Mode
	setEntryMode(module, true, false);
}

static void returnHome(TxtModule *module) {
	_sendCommand(module, 0x02);
	// Cette opération dure 1.6ms mais nous avons déjà attendu WRITE_DELAY
	// dans _sendCommand()
	Delay_Us(1600 - WRITE_DELAY);
}

static void cursorShiftControl(TxtModule *module, bool shiftDisplay, bool shiftRight) {
	uint8_t sc = shiftDisplay ? 8 : 0;
	uint8_t rl = shiftRight ? 4 : 0;
	_sendCommand(module, 0x10 | sc | rl);
}

static void setCGAddress(TxtModule *module, int address) {
	_sendCommand(module, 0x40 | (address & 0x3F));
}

static void setCursorPosition(TxtModule *module, int column, int row) {
	uint8_t rowAddr = 0;
	uint8_t colAddr = column;

	if (module->display.height > 1) {
		rowAddr = (row & 0x01) << 6;
		colAddr &= 0x3f;

		if (row > 1) {
			colAddr += 0x14;
		}
	}

	_sendCommand(module, 0x80 | rowAddr | colAddr);
}

static void writeByte(TxtModule *module, uint8_t byte) {
	commLinkWrite(&module->link, TYPE_DATA, &byte, 1);
	Delay_Us(WRITE_DELAY);
}

const struct TxtController TXT_SPLC780_CONTROLLER = {
	.initialise = initialise,
	.clear = clear,
	.returnHome = returnHome,
	.setEntryMode = setEntryMode,
	.displayControl = displayControl,
	.cursorShiftControl = cursorShiftControl,
	.setCGAddress = setCGAddress,
	.setCursorPosition = setCursorPosition,
	.writeByte = writeByte,
};
